/*******************************************************************************
 * Copyright (c) 2024-2025 R3944Realms. All rights reserved.
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 *     http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/

package com.r3944realms.dg_lab.websocket.handler.server;

import com.r3944realms.dg_lab.api.operation.ServerOperation;
import com.r3944realms.dg_lab.api.websocket.message.PowerBoxMessage;

/**
 * The type Default server operation.
 */
public class DefaultServerOperation implements ServerOperation {
    @Override
    public void HeartbeatMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void ClearMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void FeedbackMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void StrengthMessageNoticeMessage(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void StrengthMessageChangeHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void PulseClientMessageHandler(PowerBoxMessage clearMessage, PowerBoxMessage pulseMessage) {
        //NOOP
    }

    @Override
    public void PulseClientMessageHandler(PowerBoxMessage clearMessage, int delayTime, PowerBoxMessage pulseMessage) {
        //NOOP
    }

    @Override
    public void BreakConnectMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void OtherMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void ServerStartingHandler() {
        //NOOP
    }

    @Override
    public void ServerStartingErrorHandler() {

    }

    @Override
    public void ServerStartedHandler() {
        //NOOP
    }

    @Override
    public void ServerStoppingHandler() {
        //NOOP
    }

    @Override
    public void ServerStoppingErrorHandler() {

    }

    @Override
    public void ServerStoppedHandler() {
        //NOOP
    }

    @Override
    public void InactiveConnectionRemoveHandler(String clientId) {
        //NOOP
    }

    @Override
    public void ErrorMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void BindSuccessMessageHandler(PowerBoxMessage message) {
        //NOOP
    }

    @Override
    public void BindFailureMessageHandler(PowerBoxMessage message) {
        //NOOP
    }
}
